@@ -1,18 +1,26 @@
 from django.conf import settings
 from django.contrib.auth import SESSION_KEY
 from django.contrib.auth import authenticate
+from django.contrib.auth import get_user
 from django.contrib.auth import login
 from django.contrib.auth.models import User
 
 USERNAME_LENGTH = 30
 
 ALLOW_LAZY_REGISTRY = {}
+USER_AGENT_BLACKLIST = []
 
 class LazySignupMiddleware(object):
     
     def process_request(self, request):
         assert hasattr(request, 'session'), "You need to have the session app installed"
         
+        # If the user agent is one we ignore, bail early
+        request_user_agent = request.META.get('HTTP_USER_AGENT', '')
+        for user_agent in USER_AGENT_BLACKLIST:
+            if user_agent.search(request_user_agent):
+                return
+        
         # Mimic what django.core.base.BaseHandler does to resolve the request URL to a
         # callable
         from django.core import urlresolvers
@@ -40,10 +48,11 @@ class LazySignupMiddleware(object):
         # See if this view has been registered as one to skip user creation
         if not ALLOW_LAZY_REGISTRY.has_key(callback):
             return 
-                    
-        # If there's already a key in the session for the user, then we don't 
-        # need to do anything
-        if request.session.has_key(SESSION_KEY):
+        
+        # If there's already a key in the session for a valid user, then we don't 
+        # need to do anything. If the user isn't valid, then get_user will return
+        # an anonymous user.
+        if not get_user(request).is_anonymous():
             return
             
         # If not, then we have to create a user, and log them in. Set the user id
