@@ -2,6 +2,9 @@ import datetime
 
 from django.http import HttpRequest
 from django.contrib.auth import SESSION_KEY
+from django.contrib.auth import authenticate
+from django.contrib.auth import login
+from django.contrib.auth.models import AnonymousUser
 from django.contrib.auth.models import User
 from django.contrib.sessions.middleware import SessionMiddleware
 from django.contrib.sessions.models import Session
@@ -41,11 +44,25 @@ class LazyTestCase(TestCase):
     def testSessionAlreadyExists(self, mock_resolve):
         # If the user id is already in the session, this middleware should do nothing.
         f = allow_lazy(lambda: 1)
-        self.request.session[SESSION_KEY] = 1
+        user = User.objects.create_user('test', 'test@test.com', 'test')
+        self.request.user = AnonymousUser()
+        login(self.request, authenticate(username='test', password='test'))
         mock_resolve.return_value = (f, None, None)
         
         self.m.process_request(self.request)
-        self.failIf(hasattr(self.request, 'user'))
+        self.assertEqual(user, self.request.user)
+
+    @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
+    def testBadSessionAlreadyExists(self, mock_resolve):
+        # If the user id is already in the session, but the user doesn't exist,
+        # then a user should be created
+        f = allow_lazy(lambda: 1)
+        self.request.session[SESSION_KEY] = 1000
+        mock_resolve.return_value = (f, None, None)
+        
+        self.m.process_request(self.request)
+        self.assertEqual(self.request.session.session_key[:30], self.request.user.username)
+        self.assertEqual(False, self.request.user.has_usable_password())
         
     @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
     def testCreateLazyUser(self, mock_resolve):
@@ -57,6 +74,17 @@ class LazyTestCase(TestCase):
         self.assertEqual(self.request.session.session_key[:30], self.request.user.username)
         self.assertEqual(False, self.request.user.has_usable_password())
         
+    @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
+    def testBannedUserAgents(self, mock_resolve):
+        # If the client's user agent matches a regex in the banned
+        # list, then a user shouldn't be created.
+        self.request.META['HTTP_USER_AGENT'] = 'search engine'
+        f = allow_lazy(lambda: 1)
+        mock_resolve.return_value = (f, None, None)
+        self.m.process_request(self.request)
+        self.failIf(hasattr(self.request, 'user'))
+        self.assertEqual(0, len(User.objects.all()))
+        
     def testNormalView(self):
         # Calling our undecorated view should *not* create a user. If one is created, then the
         # view will set the status code to 500.
@@ -64,10 +92,12 @@ class LazyTestCase(TestCase):
         self.assertEqual(200, response.status_code)
 
     def testDecoratedView(self):
-        # Calling our undecorated view should *not* create a user. If one is created, then the
+        # Calling our undecorated view should create a user. If one is created, then the
         # view will set the status code to 500.
+        self.assertEqual(0, len(User.objects.all()))
         response = self.client.get('/lazy/')
         self.assertEqual(200, response.status_code)
+        self.assertEqual(1, len(User.objects.all()))
         
     def testRemoteExpiredUsers(self):
         # Users wihout usable passwords who don't have a current session record should be removed.
